#!/usr/bin/python
"""
    Gobject extensions to support one time/shot execution timers easily.
    Copyright (C) 2008-2009  James Shubin, McGill University
    Written for McGill University by James Shubin <purpleidea@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import gobject

__all__ = []

def false(f):
	"""evaluates the input function and returns false. useful for adding a
	one shot gobject timer. i think the code below is called a closure."""
	# create a function that calls f with whatever args it is given.
	def x(*args):
		"""generated by the false() closure function."""
		f(*args)
		return False

	# return the function that we just built.
	return x


def once_add(interval, callback, *args):
	"""run a callback in time milliseconds."""
	return gobject.timeout_add(interval, false(callback), *args)


def once_add_seconds(interval, callback, *args):
	"""run a callback in time seconds."""
	return gobject.timeout_add_seconds(interval, false(callback), *args)


# add into the gobject namespace
gobject.once_add = once_add
gobject.once_add_seconds = once_add_seconds

